-----------------------------
-- MISC HELPERS
-----------------------------

-- debugging output (uncomment to get spammed)
local function debug_print(msg)
	--d('[LocIT] '..msg)
end

local LibOrangUtils = LibStub("LibOrangUtils-1.0")

------------------------------
-- TRANSLATION-GRABBER
-----------------------------

function LocIT:GetTranslatedItemNameFromLink(link, language)
	local language = language or "EN"
	
	if(not link) then return nil end
	
	local id,name,enchant,qual,levelreq,typID = self:GetItemDataFromLink(link, language)
	
	if(LocIT_ItemData and LocIT_ItemData[language] and typID == "item") then
	
		debug_print("Checking item with ID: "..tostring(id).." for Language: "..tostring(language))
		
		if(LocIT_ItemData[language][id] and LocIT_ItemData[language][id]["name"]) then
		
			local translated_name = LocIT_ItemData[language][id]["name"]
			local template = self:GetTemplate(translated_name, language)
			local prefix = nil
			local suffix = nil
			
			-- if item is glyph
			if(translated_name == "Glyph") then
				prefix = self:GetGlyphPrefixes(levelreq, qual, language)
				suffix = self:GetEnchantmentSuffixes("Glyph", id, language)
			else
				-- only for crafting-templates, pots and jewellery
				if(template and levelreq and qual) then
					-- JEWEL
					if(template == "JEWEL") then
						prefix = self:GetJewelPrefixes(levelreq, qual, language)
						if(enchant) then suffix = self:GetEnchantmentSuffixes("Glyph", enchant, language) end
					-- POTS
					elseif(template == "POTS") then
						prefix = self:GetPotsPrefixes(levelreq, qual, language)				
					-- CRAFTING STUFF
					else
						prefix = self:GetCraftingPrefixes(template, levelreq, qual, language)
						if(enchant) then suffix = self:GetEnchantmentSuffixes("Glyph", enchant, language) end
					end
				end
			end
			
			if(prefix) then translated_name = prefix.." "..translated_name end
			if(suffix) then translated_name = translated_name.." "..suffix end
						
			return zo_strformat(SI_TOOLTIP_ITEM_NAME, translated_name)
		end
	end
	
	return nil
end

-----------------------------
-- TRANSLATION-HELPERS
-----------------------------

function LocIT:GetTranslatedItemNameFromID(id, language)
	local language = language or "EN"
	
	if(not id) then return nil end
	
	if(LocIT_ItemData and LocIT_ItemData[language] and LocIT_ItemData[language][id] and LocIT_ItemData[language][id]["name"]) then
		return zo_strformat(SI_TOOLTIP_ITEM_NAME, LocIT_ItemData[language][id]["name"])
	end
	
	return nil
end

function LocIT:GetIdsFromItemName(name, language)
	local language = language or "EN"
	local ret = {}
	
	if(not name) then return nil end
	
	if(LocIT_ItemData and LocIT_ItemData[language] ) then
		for id,item in pairs(LocIT_ItemData[language]) do
			if item["name"] == name then 
				table.insert(ret, id)
			end
		end
	end
	
	if(#ret > 0) then
		return ret
	else
		return nil
	end
end

function LocIT:GetSpecialItemPropertiesFromLink(link, language)
	local language = language or "EN"
	
	return nil
end

function LocIT:GetTemplate(name, language)
	local language = language or "EN"

	-- instead of traversing up and down lists we try it this way to save 0.000000000000001 microflops processing power
	if(LocIT_SpecialData and LocIT_SpecialData[language]) then
		if(LocIT_SpecialData[language]["JEWEL"]		and LocIT_SpecialData[language]["JEWEL"][name]	) 	then return "JEWEL"	 	end;
		if(LocIT_SpecialData[language]["POTS"]		and LocIT_SpecialData[language]["POTS"][name]	) 	then return "POTS"	 	end;
		if(LocIT_SpecialData[language]["METAL"] 	and LocIT_SpecialData[language]["METAL"][name]	) 	then return "METAL" 	end;
		if(LocIT_SpecialData[language]["CLOTH"] 	and LocIT_SpecialData[language]["CLOTH"][name]	) 	then return "CLOTH" 	end;
		if(LocIT_SpecialData[language]["LEATHER"]	and LocIT_SpecialData[language]["LEATHER"][name]) 	then return "LEATHER" 	end;
		if(LocIT_SpecialData[language]["WOOD"]		and LocIT_SpecialData[language]["WOOD"][name]	) 	then return "WOOD"	 	end;
	end
	
	return nil -- failed or non-existing
end

function LocIT:GetJewelPrefixes(level, quality, language)
	local language = language or "EN"
	local selector = tostring('JEWEL_PREFIXES')
	
	-- instead of traversing up and down lists we try it this way to save 0.000000000000001 microflops processing power
	if(LocIT_SpecialData and LocIT_SpecialData[language] and LocIT_SpecialData[language][selector]) then
		local level = tonumber(level) or nil
		local quality = tonumber(quality) or nil				
		
		if(level and level <= 15) then
			return LocIT_SpecialData[language][selector][1]
		elseif(level and level >= 16 and level <= 25) then
			return LocIT_SpecialData[language][selector][2]
		elseif(level and level >= 26 and level <= 35) then
			return LocIT_SpecialData[language][selector][3]
		elseif(level and level >= 36 and level <= 45) then
			return LocIT_SpecialData[language][selector][4]
		elseif(level and level >= 46 and level <= 50 and quality <= 50) then
			return LocIT_SpecialData[language][selector][5]
		elseif(level and level >= 50 and quality > 50 and quality <= 120) then
			local vr_lvl = tostring(quality)
				  vr_lvl = vr_lvl:sub(tostring(quality):len())
				  vr_lvl = tonumber(vr_lvl)
			
			if(vr_lvl >= 7) then
				return LocIT_SpecialData[language][selector][6]
			end	
		end
	end
	
	return nil -- failed or non-existing
end

function LocIT:GetPotsPrefixes(level, quality, language)
	local language = language or "EN"
	local selector = tostring('POTS_PREFIXES')
	
	-- instead of traversing up and down lists we try it this way to save 0.000000000000001 microflops processing power
	if(LocIT_SpecialData and LocIT_SpecialData[language] and LocIT_SpecialData[language][selector]) then
		local level = tonumber(level) or nil
		local quality = tonumber(quality) or nil		
		
		if(level and level < 12) then
			return LocIT_SpecialData[language][selector][1]
		elseif(level and level >= 12 and level < 16) then
			return LocIT_SpecialData[language][selector][2]
		elseif(level and level >= 16 and level < 21) then
			return LocIT_SpecialData[language][selector][3]
		elseif(level and level >= 21 and level < 26) then
			return LocIT_SpecialData[language][selector][4]
		elseif(level and level >= 26 and level < 31) then
			return LocIT_SpecialData[language][selector][5]
		elseif(level and level >= 31 and level < 36) then
			return LocIT_SpecialData[language][selector][6]
		elseif(level and level >= 36 and level < 41) then
			return LocIT_SpecialData[language][selector][7]
		elseif(level and level >= 41 and level < 46) then
			return LocIT_SpecialData[language][selector][8]
		elseif(level and level >= 46 and level <= 50 and quality <= 50) then
			return LocIT_SpecialData[language][selector][9]					
		elseif(level and level >= 50 and quality > 50 and quality <= 120) then
			local vr_lvl = tostring(quality)
				  vr_lvl = vr_lvl:sub(tostring(quality):len())
				  vr_lvl = tonumber(vr_lvl)
			
			if(vr_lvl <= 7) then
				return LocIT_SpecialData[language][selector][10]
			end
		end
	end
	
	return nil -- failed or non-existing
end

function LocIT:GetCraftingPrefixes(material, level, quality, language)
	local language = language or "EN"
	local selector = tostring(material..'_PREFIXES')
	
	-- instead of traversing up and down lists we try it this way to save 0.000000000000001 microflops processing power
	if(LocIT_SpecialData and LocIT_SpecialData[language] and LocIT_SpecialData[language][selector]) then
		local level = tonumber(level) or nil
		local quality = tonumber(quality) or nil		
		
		if(level and level <= 15) then
			return LocIT_SpecialData[language][selector][1]
		elseif(level and level >= 16 and level <= 25) then
			return LocIT_SpecialData[language][selector][2]
		elseif(level and level >= 26 and level <= 35) then
			return LocIT_SpecialData[language][selector][3]
		elseif(level and level >= 36 and level <= 45) then
			return LocIT_SpecialData[language][selector][4]
		elseif(level and level >= 46 and level <= 50 and quality <= 50) then
			return LocIT_SpecialData[language][selector][5]
		elseif(level and level >= 50 and quality > 50 and quality <= 120) then
			local vr_lvl = tostring(quality)
				  vr_lvl = vr_lvl:sub(tostring(quality):len())
				  vr_lvl = tonumber(vr_lvl)
			
			if(vr_lvl <= 3) then
				return LocIT_SpecialData[language][selector][6]
			elseif(vr_lvl >= 4 and vr_lvl <= 6) then
				return LocIT_SpecialData[language][selector][7]
			elseif(vr_lvl >= 7 and vr_lvl <= 8) then
				return LocIT_SpecialData[language][selector][8]
			elseif(vr_lvl >= 9) then
				return LocIT_SpecialData[language][selector][9]
			end	
		end
	end
	
	return nil -- failed or non-existing
end

function LocIT:GetGlyphPrefixes(level, quality, language)
	local language = language or "EN"
	local selector = tostring('GLYPH_PREFIXES')
	
	-- instead of traversing up and down lists we try it this way to save 0.000000000000001 microflops processing power
	if(LocIT_SpecialData and LocIT_SpecialData[language] and LocIT_SpecialData[language][selector]) then
		local level = tonumber(level) or nil
		local quality = tonumber(quality) or nil		
		
		if(level <= 11 and quality <= 50) then 
			if(LocIT_SpecialData[language][selector][1]) then return LocIT_SpecialData[language][selector][1] end
		
		--|HFFFFFF:item:26844:2:12:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h[Petty Glyph of Shock]|h		
		elseif(level >= 12 and level < 16 and quality <= 50) then
			if(LocIT_SpecialData[language][selector][3]) then return LocIT_SpecialData[language][selector][2] end
		
		--|HFFFFFF:item:26844:2:16:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h[Slight Glyph of Shock]|h
		--|HFFFFFF:item:26844:2:17:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h[Slight Glyph of Shock]|h
		--|HFFFFFF:item:45871:2:19:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h[Slight Glyph of Reduce Feat Cost]|h
		elseif(level >= 15 and level < 22 and quality <= 50) then
			if(LocIT_SpecialData[language][selector][2]) then return LocIT_SpecialData[language][selector][3] end
			
		--|HFFFFFF:item:26589:2:22:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h[Minor Glyph of Stamina Regen]|h	
		--|HFFFFFF:item:26588:2:23:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h[Minor Glyph of Stamina]|h	
		elseif(level >= 22 and level < 26 and quality <= 50) then
			if(LocIT_SpecialData[language][selector][5]) then return LocIT_SpecialData[language][selector][4] end	
			
		--|HFFFFFF:item:26844:2:27:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h[Lesser Glyph of Shock]|h	
		elseif(level >= 25 and level < 31 and quality <= 50) then
			if(LocIT_SpecialData[language][selector][4]) then return LocIT_SpecialData[language][selector][5] end
		
		--|HFFFFFF:item:45870:2:31:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h[Moderate Glyph of Reduce Spell Cost]|h
		--|HFFFFFF:item:26588:2:32:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h[Moderate Glyph of Stamina]|h			
		elseif(level >= 31 and level < 40 and quality <= 50) then
			if(LocIT_SpecialData[language][selector][6]) then return LocIT_SpecialData[language][selector][6] end
			
		--|HFFFFFF:item:45872:2:45:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h[Strong Glyph of Bashing]|h
		elseif(level >= 40 and level <= 45 and quality <= 50) then
			if(LocIT_SpecialData[language][selector][9]) then return LocIT_SpecialData[language][selector][7] end	
			
		--|HFFFFFF:item:26845:2:48:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h[Great Glyph of Crushing]|h	
		elseif(level >= 46 and level <= 50 and quality <= 50) then
			if(LocIT_SpecialData[language][selector][8]) then return LocIT_SpecialData[language][selector][8] end
			
		elseif(level >= 50 and quality > 50 and quality <= 120) then
		
			local vr_lvl = tostring(quality)
				  vr_lvl = vr_lvl:sub(tostring(quality):len())
				  vr_lvl = tonumber(vr_lvl)
				  			
			if(vr_lvl <= 3) then
				return LocIT_SpecialData[language][selector][9]
			elseif(vr_lvl >= 4 and vr_lvl <= 6) then
				return LocIT_SpecialData[language][selector][10]
			elseif(vr_lvl >= 7 and vr_lvl <= 8) then
				return LocIT_SpecialData[language][selector][11]
			elseif(vr_lvl >= 9) then
				return LocIT_SpecialData[language][selector][12]
			end		
		end	
	end
	
	return nil
end

function LocIT:GetEnchantmentSuffixes(template_name, enchant, language)
	local language = language or "EN"
	local selector = tostring(template_name:upper()..'_SUFFIXES')
	local enchant = tonumber(enchant) or nil
	
	-- instead of traversing up and down lists we try it this way to save 0.000000000000001 microflops processing power
	if(LocIT_SpecialData and LocIT_SpecialData[language] and LocIT_SpecialData[language][selector]) then	
		if(LocIT_SpecialData[language][selector][enchant]) then return LocIT_SpecialData[language][selector][enchant] end
	end
	
	return nil
end

-----------------------------
-- ITEM HELPER
-----------------------------

function LocIT:GetItemDataFromLink(link)
	local name,col,typID,id,qual,levelreq,enchant,ench1,ench2,un1,un2,un3,un4,un5,un6,un7,un8,un9,style,un10,bound,charge,un11=ZO_LinkHandler_ParseLink(link)		
	
	id = tonumber(id)
	name = zo_strformat(SI_TOOLTIP_ITEM_NAME, name)
		
	return id,name,enchant,qual,levelreq,typID,ench1,ench2,col,un1,un2,un3,un4,un5,un6,un7,un8,un9,style,un10,bound,charge,un11
end

-----------------------------
-- DUMPING STUFF
-----------------------------

local function LocIT_GetItemDumpStr(id,name,enchant,qual,levelreq,link)
	name = name:match("%[(.-)%]") or name
	
	local ret = '['..tostring(id)..'] = {["name"] = "'..tostring(name)..'"'
	
	if(enchant) 	then ret = ret..', ["enchant"] = "'..tostring(enchant)..'"' 	end
	if(qual) 		then ret = ret..', ["qual"] = "'..tostring(qual)..'"' 			end
	if(levelreq) 	then ret = ret..', ["levelreq"] = "'..tostring(levelreq)..'"' 	end
	if(link) 		then ret = ret..', ["link"] = "'..tostring(link)..'"' end
	
	ret = ret..'}'
	
	return ret
end

local function LocIT_GetItemDumpStrFromLink(link)
	local id,name,enchant,qual,levelreq = LocIT:GetItemDataFromLink(link)	
	return LocIT_GetItemDumpStr(id,name,enchant,qual,levelreq,link)
end

local function LocIT_GetItemsFromBags()
	local ret = setmetatable({}, {
		__newindex = function(self,k,v)
			rawset(self,k,v)
			rawset(self,v.id,k)
		end
	})
		
	for bagId = 1, GetMaxBags() do
		local _, maxSlots = GetBagInfo(bagId)
		for slotId = 1, maxSlots do
			
			local link = GetItemLink(bagId, slotId)
						
			if(link and link ~= "") then	
				local id,name,enchant,qual,levelreq = LocIT:GetItemDataFromLink(link)
				
				if(not ret[id]) then -- TODO: better way of figuring out duplicates
					ret[#ret+1] = { ["id"] = id, ["name"] = name, ["enchant"] = enchant, ["qual"] = qual, ["levelreq"] = levelreq, ["link"] = link }
				end
			end
		end
	end

	table.sort(ret, function(a,b) return a.id < b.id end)
	
	return ret
end

local function LocIT_GetItemsFromGuildBank()
	local ret = setmetatable({}, {
		__newindex = function(self,k,v)
			rawset(self,k,v)
			rawset(self,v.id,k)
		end
	})

	for i,data in ipairs(ZO_GuildBankBackpack.data) do
		--SelectGuildBank(guildId)
		data = data.data
		
		local link = GetItemLink(BAG_GUILDBANK, data.slotIndex)
				
		if(link and link ~= "") then
			local id,name,enchant,qual,levelreq = LocIT:GetItemDataFromLink(link)
			
			if(not ret[id]) then -- TODO: better way of figuring out duplicates
				ret[#ret+1] = { ["id"] = id, ["name"] = name, ["enchant"] = enchant, ["qual"] = qual, ["levelreq"] = levelreq, ["link"] = link }
			end
		end
	end

	table.sort(ret, function(a,b) return a.constructed_id < b.constructed_id end)
	
	return ret
end

local function LocIT_GetItemDifferencesFrom(source)
	--local source = source or "inventory"
	local unkown = ""
	local different = ""
	local data = nil
	
	if(source == "guild") then
		data = LocIT_GetItemsFromGuildBank()
	elseif(source == "inventory") then
		data = LocIT_GetItemsFromBags()
	else
		-- COLLECTOR
	end
	
	if(not data or data == "") then do return nil end end
	
	for _,item in ipairs(data) do
	
		local translated_name = LocIT:GetTranslatedItemNameFromID(item.id)
		
		if(not translated_name) then
			unkown = unkown..'['..tostring(item.id)..'] = {["name"] = "'..tostring(item.name)..'", ["enchant"] = "'..tostring(item.enchant)..'"},'.."\n"
		elseif(translated_name ~= item.name) then
			different = different..'['..tostring(item.id)..'] = {["name"] = "'..tostring(item.name)..'", ["enchant"] = "'..tostring(item.enchant)..'"},'.."\n"
		end
	end
	
	return unkown, different
end

function LocIT_DumpDifferences(source)
	local source = source or "inventory"
	local unkown, different
	local text = ""
	
	unkown, different = LocIT_GetItemDifferencesFrom(source)

	if(unkown and unkown ~= "") then
		text = text.."Unkown Items:\n"..unkown
	end
	
	if(different and different ~= "") then
		text = text.."\nDifferent Items:\n"..different
	end
	
	
end

--color--type--id--qual--levelreq--enchant
--|HFFFFFF:item:27037:2:10:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h[tincture of Magicka]|h
--|HFFFFFF:item:27037:2:10:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h[Tincture of Magicka]|h
--|H2DC50E:item:1693:9:21:0:0:0:0:0:0:0:0:0:0:0:0:1:0:0:0:0|h[Murdered Mason's Boots]|h
--|HFFFFFF:item:43573:2:18:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h[Slight Glyph of Absorb Health]|h

--|H3A92FF:item:45858: 4:16:26588:4:16:0:0:0:0:0:0:0:0:0: 6:0:0:0:0|h[Witwenhandschuhe der Ausdauer]|h
--|H3A92FF:item:45147: 4:17:26580:4:17:0:0:0:0:0:0:0:0:0: 7:0:0:0:0|h[Halblederstiefel des Lebens]|h
--|HFFFFFF:item:26580: 2:13:    0:0: 0:0:0:0:0:0:0:0:0:0: 7:0:0:0:0|h[Petty Glyph of Health]|h

--|HFFFFFF:item:43538:20:10:26580:2:13:0:0:0:0:0:0:0:0:0: 1:1:0:0:0|h[Iron Sabatons of Health]|h
--|HFFFFFF:item:43538:20:10:    0:0: 0:0:0:0:0:0:0:0:0:0: 1:1:0:0:0|h[Iron Sabatons]|h
--|H3A92FF:item:45147: 4:17:26580:4:17:0:0:0:0:0:0:0:0:0: 7:0:0:0:0|h[Halblederstiefel des Lebens]|h
--|HFFFFFF:item:43544:20:10:26580:2:13:0:0:0:0:0:0:0:0:0:34:1:1:0:0|h[Homespun Shoes of Health]|h
--|HFFFFFF:item:43546:20:10:26582:2:13:0:0:0:0:0:0:0:0:0: 1:1:0:0:0|h[Homespun Breeches of Magicka]|h

--|H3A92FF:item:45858:30: 1:    0:3: 1:0:0:0:0:0:0:0:0:0: 1:1:0:0:0|h[Witwenhandschuhe]|h
--|HFFFFFF:item:43545:30: 1:    0:3: 1:0:0:0:0:0:0:0:0:0: 1:1:0:0:0|h[Homespun Gloves]|h

-----------------------------
-- TRANSLATION-WORKER(S)
-----------------------------

-- translate item-links
function LocIT:TranslateItemLinks(text, language)
	local language = language or "EN"

	for link in string.gfind(text, "(|H(.-):(.-)|h(.-)|h)") do
		if(link and type(link) == "string") then
			local link = tostring(link)
					
			debug_print("Checking item for translation: "..tostring(link))
			
			local translated_name = self:GetTranslatedItemNameFromLink(link, language);
			
			if(translated_name and type(translated_name) == "string") then 
				debug_print("Translation found: "..translated_name) 
				local old_name = link:match("|h%[(.-)%]|h")
				
				--debug_print("Trying to replace: "..tostring(old_name))
				if(old_name) then 
					-- escape magic patterns
					old_name = LibOrangUtils:str_escape(old_name)
				
					text = LibOrangUtils:str_replace(text, '%['..old_name..'%]', '['..translated_name..']', false) 
				else
					debug_print("Couldn't get the old_name for the Link: "..tostring(link))
				end
			end
		end
    end
	
	return text
end

-----------------------------
-- COMMANDS
-----------------------------

function LocIT_ParseCommands(cmd_str)
	local ret = LibOrangUtils:str_split(cmd_str, "%s")
	d(ret)
	return ret
end

function LocIT_OnCommand_locit(cmd_str)
	local cmd_str = LibOrangUtils:str_cleanup(cmd_chain)
	
	if(cmd_str) then
		local cmd_chain = LocIT_ParseCommands(cmd_str)
	else -- no subcommands supplied
	
	end
	
	debug_print("/locit - subcommands: "..tostring(cmd_str))
end

function LocIT_OnCommand_dump_items(cmd_str)
	local cmd_str = LibOrangUtils:str_cleanup(cmd_str)
	
	if(cmd_str) then
		local cmd_chain = LocIT_ParseCommands(cmd_str)
	else -- no subcommands supplied
	
	end
	
	debug_print("/dump-items - subcommands: "..tostring(cmd_str))
end

-----------------------------
-- EVENTS
-----------------------------

-- OnChatMessage event
local function LocIT_OnChatMessage(messageType, fromName, text) 
	local ChannelInfo = ZO_ChatSystem_GetChannelInfo()	
	local channelInfo = ChannelInfo[messageType]

	text = LocIT:TranslateItemLinks(text)
	
	if (channelInfo and channelInfo.format) then
		local channelLink = nil
		if (channelInfo.channelLinkable) then
			local channelName = GetChannelName(channelInfo.id)
			channelLink = ZO_LinkHandler_CreateChannelLink(channelName)
		end
        local fromLink = fromName
		if (channelInfo.playerLinkable) then
			fromLink = ZO_LinkHandler_CreatePlayerLink(fromName)
		end
        if (channelLink) then
			local channelName = nil
            result = zo_strformat(channelInfo.format, channelLink, fromLink, text)
        else
			result = zo_strformat(channelInfo.format, fromLink, text)
		end
		return result, channelInfo.saveTarget
    end			
end

 
local function LocIT_OnLinkInsert(link)
	-- Not yet tested so it 's disabled for now
	link = LocIT:TranslateItemLinks(link)
end

local function LocIT_OnLinkClicked(link, button, text, color, linkType, ...)
	-- RIGHTCLICK
	if(button == 2 and linkType == ITEM_LINK_TYPE) then
		ClearMenu()
        AddMenuItem("Dump Item", function() 
			local dump_str = LocIT_GetItemDumpStrFromLink(link)	
			LibOrangUtils:CopyToClipboard(dump_str)			
			--debug_print(dump_str)
		end)
        ShowMenu(nil, 1)
	--debug_print("Rightclick on LINK")
	end
end

-- OnPlayerActivated event
local function LocIT_OnPlayerActivated()
	-- :(
end

-- OnAddOnLoaded event
local function LocIT_OnAddonLoaded(event, addon)
	if(addon == "LocalizedItems") then
		zo_callLater(function()
			debug_print("Loading "..tostring(LocIT.id).." (Build #"..tostring(LocIT.build)..") ...")
					
			local CHAT_HANDLERS = ZO_ChatSystem_GetEventHandlers()
			local OnChatMessageReceived = CHAT_HANDLERS[EVENT_CHAT_MESSAGE_CHANNEL]
			local X4D_LibAntiSpam = LibStub:GetLibrary('LibAntiSpam', true)
			
			if(X4D_Chat) then
				OnChatMessageReceived = X4D_Chat.OnChatMessageReceived
				
				debug_print("Found X4D Chat OnChatMessageReceived. Hooking it ...")				
					
				X4D_Chat.OnChatMessageReceived = function(messageType, fromName, text, ...) 
					text = LocIT:TranslateItemLinks(text)
					return OnChatMessageReceived(messageType, fromName, text, ...)
				end				
			elseif(not X4D_Chat and X4D_LibAntiSpam) then
				OnChatMessageReceived = X4D_LibAntiSpam.OnChatMessageReceived
				
				debug_print("Found X4D LibAntiSpam OnChatMessageReceived. Hooking it ...")
				
				X4D_LibAntiSpam.OnChatMessageReceived = function(messageType, fromName, text, ...) 
					text = LocIT:TranslateItemLinks(text)
					return OnChatMessageReceived(messageType, fromName, text, ...)
				end
			elseif(OnChatMessageReceived) then
				debug_print("Chat already has an OnChatMessageReceived Event. Hooking it ...")
				
				local function Hook_OnChatMessage(messageType, fromName, text, ...)
					text = LocIT:TranslateItemLinks(text)
					return OnChatMessageReceived(messageType, fromName, text, ...)
				end
			
				ZO_ChatSystem_AddEventHandler(EVENT_CHAT_MESSAGE_CHANNEL, Hook_OnChatMessage)
			else
				debug_print("Chat has no OnChatMessageReceived Event yet, adding one ourselves.")
				ZO_ChatSystem_AddEventHandler(EVENT_CHAT_MESSAGE_CHANNEL, LocIT_OnChatMessage)
			end
		
			-- Register Commands
			-- SLASH_COMMANDS["/locit"] 		= LocIT_OnCommand_locit
			-- SLASH_COMMANDS["/dump-items"]	= LocIT_OnCommand_dump_items
			debug_print("LocalizedItems loaded.")
			
			EVENT_MANAGER:UnregisterForEvent("LocIT_EVENT_PLAYER_ACTIVATED", EVENT_PLAYER_ACTIVATED) -- only do it once, and then unregister
		end, 1)
	end
end

-----------------------------
-- EVENT-REGISTRATION 
-----------------------------

EVENT_MANAGER:RegisterForEvent("LocIT_EVENT_ADD_ON_LOADED", EVENT_ADD_ON_LOADED , LocIT_OnAddonLoaded)
--EVENT_MANAGER:RegisterForEvent("LocIT_EVENT_PLAYER_ACTIVATED", EVENT_PLAYER_ACTIVATED, LocIT_OnPlayerActivated)
--LINK_HANDLER:RegisterCallback(LINK_HANDLER.INSERT_LINK_EVENT, LocIT_OnLinkInsert)
LINK_HANDLER:RegisterCallback(LINK_HANDLER.LINK_CLICKED_EVENT, LocIT_OnLinkClicked)