local LibITCol = LibStub:NewLibrary("LibItemCollector-1.0", 1)

if not LibITCol then
	do return end -- already loaded and no upgrade necessary
end

local function table_copy(t)
  local u = { }
  for k, v in pairs(t) do u[k] = v end
  return u
end

LibITCol.Collector = {}

function LibITCol.Collector:New() 
	local ret = {}
	
	ret.items = {}
	
	setmetatable(ret, { __index = LibITCol.Collector })

	return ret
end

function LibITCol.Collector:AddItem(id, name, enchant, lvl, quality, link)
	local enchant 	= enchant or nil 
	local lvl 		= lvl or nil
	local quality 	= quality or nil
	local link 		= link or nil
	local ret 		= nil
	
	local index = self:GetItemIndexByID(id)
	
	-- update old item
	if(index) then
		d("Update Item")
		self.items[index] = { ["id"] = id, ["name"] = name, ["enchant"] = enchant, ["lvl"] = lvl, ["quality"]=quality, ["link"] = link }
		ret = self.items[index]
	-- insert new item
	else
		d("New Item")
		local new_index = #self.items + 1
		self.items[new_index] = { ["id"] = id, ["name"] = name, ["enchant"] = enchant, ["lvl"] = lvl, ["quality"]=quality, ["link"] = link }
		ret = self.items[new_index]
	end
	
	return ret
end

function LibITCol.Collector:GetItemByID(id)
	local index = nil
	local id = tonumber(id)
	
	for i,item in ipairs(self.items) do
		if(type(id) == "number" and type(item) == "table" and item.id and item.id == id) then index = i end
	end
	
	return self.item[index]
end

function LibITCol.Collector:GetItemIndexByID(id)
	local index = nil
	local id = tonumber(id)
	
	for i,item in ipairs(self.items) do
		if(type(id) == "number" and type(item) == "table" and item.id and item.id == id) then index = i end
	end
	
	return index
end

function LibITCol.Collector:GetItemIndexesByName(name)
	local ret = {}
	local name = tostring(name)
	
	for i,item in ipairs(self.items) do
		if(type(item) == "table" and item.name and item.name == name) then ret[#ret +1] = i end
	end
	
	return ret
end

function LibITCol.Collector:GetItemsByName(name)
	local ret = {}
	local name = tostring(name)
	
	for i,item in ipairs(self.items) do
		if(type(item) == "table" and item.name and item.name == name) then ret[#ret +1] = item end
	end
	
	return ret
end

function LibITCol.Collector:DeleteItemByID(id)
	local index = self:GetItemIndexByID(id)

	if(not index) then do return false end end
	
	self.items[index] = nil
	
	self:ReIndexItems()
	
	return true
end

function LibITCol.Collector:DeleteItemsByName(name)
	local items = self:GetItemIndexesByName(name)

	for _,index in ipairs(items) do
		self.items[index] = nil 
	end
	
	self:ReIndexItems()
	
	return true
end

function LibITCol.Collector:GetAllItems()
	return self.items 
end

function LibITCol.Collector:ReIndexItems()
	local old_items = table_copy(self.items)
	
	self:DeleteAllItems()
	
	for _,item in pairs(old_items) do
		local index = #self.items + 1
		self.items[index] = table_copy(item)
	end
	
	for k,v in pairs(old_items) do old_items[k]=nil end
	old_items = nil
end

function LibITCol.Collector:DeleteAllItems()
	for k,v in pairs(self.items) do self.items[k]=nil end
	
	self.items = {}
end